#include "CalcParPanel.h"
#include "CalcWindow.h"
#include "three_bodycalc.h"


CCalcParPanel::CCalcParPanel( CDrawField* f, QWidget *parent)
:QWidget(parent)
{
	field = f;
	init();
	
}

CCalcParPanel::~CCalcParPanel(){
}


void CCalcParPanel::init(){
	//QVBoxLayout *layout = new QVBoxLayout;

	QFormLayout *flayout = new QFormLayout;

        pcmbSolver = new QComboBox;
        pcmbSolver->insertItem(0, tr("Runge Kutta2"));
        pcmbSolver->insertItem(1, tr("Euler"));
        pcmbSolver->insertItem(2, tr("Runge Kutta4"));
        connect(pcmbSolver, SIGNAL(currentIndexChanged(int)), this, SLOT(changeSolver(int)));

        ple_nstep = new QLineEdit;
	ple_px = new QLineEdit;
	ple_factor = new QLineEdit;
	ple_av1 = new QLineEdit;
	ple_av2 = new QLineEdit;
	ple_av3 = new QLineEdit;
        ple_avtotal = new QLineEdit;
		
	btnZmP = new QPushButton(tr("+"));
	btnZmM = new QPushButton(tr("-"));
	btnUpdate = new QPushButton(tr("update"));
	connect(btnUpdate, SIGNAL(clicked()), this, SLOT(simUpdate()));
	connect(btnZmP, SIGNAL(clicked()), field, SLOT(zoomIn()));
	connect(btnZmM, SIGNAL(clicked()), field, SLOT(zoomOut()));

        flayout->addRow(new QLabel(tr("Solver")));
        flayout->addRow(pcmbSolver);
        flayout->addRow(new QLabel(tr("Nstep :")), ple_nstep);

	flayout->addRow(new QLabel(tr("Initial Position(xp3)")));
	flayout->addRow(new QLabel(tr("xp3 :")), ple_px);
	
	flayout->addRow(new QLabel(tr("Factor(rate of mass)")));
	flayout->addRow(new QLabel(tr("Factor :")), ple_factor);
	
        flayout->addRow(btnUpdate);

        flayout->addRow(new QLabel(tr("")));
	flayout->addRow(new QLabel(tr("Zoom")));
	flayout->addRow(btnZmP, btnZmM);


	flayout->addRow(new QLabel(tr("STD Of Conservation Field(anguler momentum)")));
	flayout->addRow(new QLabel(tr("Star :")), ple_av1);
	flayout->addRow(new QLabel(tr("Planet1 :")), ple_av2);
	flayout->addRow(new QLabel(tr("Planet2 :")), ple_av3);
        flayout->addRow(new QLabel(tr("Total :")), ple_avtotal);

	
	setLayout(flayout);
}

void CCalcParPanel::updateParam(){
        ple_px->setText(QString::number(CCalcEngine::Xp3()));
        ple_factor->setText(QString::number(CCalcEngine::Factor()));
        ple_nstep->setText(QString::number(CCalcEngine::NStep()));
}

void CCalcParPanel::changeSolver(int id)
{
    //qDebug(QString::number(id).toStdString().c_str());

    if(id==0){
        pengine=prk2;

    }else if(id==1){
        pengine=peuler;

    }else{
        pengine=prk4;
    }

}

void CCalcParPanel::updateAMInfo(){
        am_list_t* am_l = CCalcEngine::getAMList();
	ple_av1->setText(QString::number(am_l->am[0]));
	ple_av2->setText(QString::number(am_l->am[1]));
	ple_av3->setText(QString::number(am_l->am[2]));
        ple_avtotal->setText(QString::number(am_l->am[3]));
}


void CCalcParPanel::simUpdate()
{
        CCalcEngine::setParam(ple_px->text().toDouble(), ple_factor->text().toDouble());
        CCalcEngine::NStep(ple_nstep->text().toInt());
	pengine->Calculation();
	
	updateAMInfo();	
	field->update();
}
