#include "three_bodycalc.h"
#include <math.h>
#include <stdlib.h>
#include <iostream>

int CCalcEngine::maxtime;
int CCalcEngine::nstep;
int CCalcEngine::count;
double CCalcEngine::dtmin;
double CCalcEngine::dt;
double CCalcEngine::time;
//double CCalcEngine::end_time;
double CCalcEngine::xp[3];
double CCalcEngine::yp[3];
double CCalcEngine::vx[3];
double CCalcEngine::vy[3];
double CCalcEngine::mp[3];
double CCalcEngine::xg;
double CCalcEngine::yg;
double CCalcEngine::vxg;
double CCalcEngine::vyg;
double CCalcEngine::vp;
am_list_t CCalcEngine::am_list;
double CCalcEngine::factor;
data_list_t CCalcEngine::data_list[10000];

CCalcEngine::CCalcEngine()
:PI_ (3.1415926535898)
{
        nstep=1.0e06;
        maxtime=3.0e5;
        
}

void CCalcEngine::setParam(double xp3, double factor_)
{
	factor=factor_;
	time = 0;

	/*//データ列の初期化
	for(int ct=0; ct < 10000; ct++){
		for(int ip=0; ip < 3; ip++){
			data_list[ct].xp[ip]=xp[ip];
              	data_list[ct].yp[ip]=yp[ip];
			data_list[ct].ag[ip]	
		}
	}*/	
	//qDebug("%f,%f", xp3,factor_);




	//質量を初期化	
	mp[0]=1;
	mp[1]=9.5479e-4*factor;
	mp[2]=2.859e-4*factor;	

	//3天体の初期位置を設定
	xp[0]=0;
	yp[0]=0;
	
	xp[1]=1.0;
	yp[1]=0;

	xp[2]=xp3;
	yp[2]=0;

	//3天体の惑星の初期速度を設定
	vx[0]=0;
	vy[0]=0;

	vx[1]=0;
        vy[1]=sqrt(mp[0]+mp[1])/sqrt(abs((long)xp[1]));

	vx[2]=0;
        vy[2]=sqrt(mp[0]+mp[2])/sqrt(abs((long)xp[2]));;

	//重心を求める
	double mtotal=0;	
	xg=yg=vxg=vyg=0;


	for(int ip=0; ip < 3; ip++){
		xg=xg+mp[ip]*xp[ip];
		yg=yg+mp[ip]*yp[ip];
		vxg=vxg+mp[ip]*vx[ip];
		vyg=vyg+mp[ip]*vy[ip];
		mtotal = mtotal + mp[ip];
	}
	
	xg=xg/mtotal;
	yg=yg/mtotal;
	vxg=vxg/mtotal;
	vyg=vyg/mtotal;
	
	//重心を原点へ
	for(int ip=0; ip < 3; ip++){
		xp[ip]-=xg;
		yp[ip]-=yg;
		vx[ip]-=vxg;
		vy[ip]-=vyg;	
        }

		
}


void CCalcEngine::CalcStDeviation()
{
        double ave[4];
        double std[4];

        for(int jp=0; jp<4;jp++){
		ave[jp] = 0;
		std[jp] = 0;
	}

	for(int ip=0; ip < count; ip++){
		for(int jp=0; jp < 3; jp++){
			ave[jp] += data_list[ip].ag[jp];
		}
	}

        ave[3] = (ave[0]+ave[1]+ave[2])/count;
	
	for(int jp=0; jp<3;jp++){
		ave[jp] = ave[jp]/count;
	}

	for(int ip=0; ip < count; ip++){
		for(int jp=0; jp<3; jp++){
			std[jp] += pow(ave[jp] - data_list[ip].ag[jp], 2);
		}

                std[3] += pow(ave[3] - (data_list[ip].ag[0]+ data_list[ip].ag[1] + data_list[ip].ag[2]), 2);
	}
	
        for(int jp=0; jp<4;jp++){
		am_list.am[jp] = std[jp]/count;
	}	
}

CThreeBodyEuler::CThreeBodyEuler()
:CCalcEngine()
{
	

}

CThreeBodyEuler::~CThreeBodyEuler(){
	//CCalcEngine::~CCalcEngine();
}


void CThreeBodyEuler::Calculation(){
    double fr, fx[3][3], fy[3][3];
    double rp[3][3];
    //double dth, frh, xph[3], yph[3], rph[3][3], vxh[3], vyh[3], fxh[3][3], fyh[3][3];
    dt=0;
    count=0;

    for(int step=1; step < nstep; step++){
        dtmin=1.0e11;

        for(int ip=0; ip<3; ip++){
            for(int jp=0; jp<3; jp++){
                if(jp != ip){
                    rp[jp][ip]=sqrt(pow(xp[jp]-xp[ip],2)+pow(yp[jp]-yp[ip],2));
                    fr=-mp[ip]/pow(rp[jp][ip],2);
                    fx[jp][ip]=fr*(xp[jp]-xp[ip])/rp[jp][ip];
                    fy[jp][ip]=fr*(yp[jp]-yp[ip])/rp[jp][ip];
                    vp = sqrt(pow(vx[jp]-vx[ip],2)+pow(vy[jp]-vy[ip],2));
                    dtmin=min(dtmin, rp[jp][ip]/vp);

                }else{
                    rp[jp][ip]=0;
                    fx[jp][ip]=0;
                    fy[jp][ip]=0;
                }
            }
        }


        dt=2*PI_*dtmin*1.001e-3;

        for(int ip=0; ip<3; ip++){
            double fxtotal, fytotal;
            fxtotal=fytotal=0;

            for(int jp=0; jp<3; jp++){
               fxtotal+=fx[ip][jp];
                fytotal+=fy[ip][jp];
            }

            xp[ip]=xp[ip] + vx[ip]*dt;
            yp[ip]=yp[ip] + vy[ip]*dt;
            vx[ip]=vx[ip]+fxtotal*dt;
            vy[ip]=vy[ip]+fytotal*dt;
        }

        time+=dt;

        if(step%10000==1){
            //std::cout << step << ":" << xp[1] << "," << yp[1] << std::endl;

            if(count < 10000){
                 for(int ip=0; ip<3; ip++){
                    data_list[count].xp[ip]=xp[ip];
                    data_list[count].yp[ip]=yp[ip];
                      data_list[count].ag[ip]= sqrt(xp[ip]*xp[ip] + yp[ip]*yp[ip]) * sqrt(vx[ip]*vx[ip] + vy[ip]*vy[ip]);
                  }
             }
            count++;
        }


     }

     CalcStDeviation();
}

/////////////////////////////////////////////
//2次ルンゲクッタ
//
///////////////////////////////////////////
CThreeBodyRK2::CThreeBodyRK2()
:CCalcEngine()
{
}

CThreeBodyRK2::~CThreeBodyRK2()
{
}

//ルンゲクッタ2次によるソルバ
void CThreeBodyRK2::Calculation()
{
    double fr, fx[3][3], fy[3][3];
    double rp[3][3];
    double dth, frh, xph[3], yph[3], rph[3][3], vxh[3], vyh[3], fxh[3][3], fyh[3][3];
    dt=0;
    count=0;

    for(int step=1; step < nstep; step++){
        dtmin=1.0e11;

        for(int ip=0; ip<3; ip++){
            for(int jp=0; jp<3; jp++){
                if(jp != ip){
                    rp[jp][ip]=sqrt(pow(xp[jp]-xp[ip],2)+pow(yp[jp]-yp[ip],2));
                    fr=-mp[ip]/pow(rp[jp][ip],2);
                    fx[jp][ip]=fr*(xp[jp]-xp[ip])/rp[jp][ip];
                    fy[jp][ip]=fr*(yp[jp]-yp[ip])/rp[jp][ip];
                    vp = sqrt(pow(vx[jp]-vx[ip],2)+pow(vy[jp]-vy[ip],2));
                    dtmin=min(dtmin, rp[jp][ip]/vp);

                }else{
                    rp[jp][ip]=0;
                    fx[jp][ip]=0;
                    fy[jp][ip]=0;
                }
            }
        }


        dt=2*PI_*dtmin*1.001e-3;
        dth = dt*0.5e0;

        for(int ip=0; ip<3; ip++){
            double fxtotal, fytotal;
            fxtotal=fytotal=0;

            for(int jp=0; jp<3; jp++){
               fxtotal+=fx[ip][jp];
                fytotal+=fy[ip][jp]; 
            }

            xph[ip]=xp[ip] + vx[ip]*dth;
            yph[ip]=yp[ip] + vy[ip]*dth;
            vxh[ip]=vx[ip]+fxtotal*dth;
            vyh[ip]=vy[ip]+fytotal*dth;
        }

        for(int ip=0; ip<3; ip++){
            for(int jp=0; jp<3; jp++){
                if(jp != ip){
                    rph[jp][ip]=sqrt(pow(xph[jp]-xph[ip],2)+pow(yph[jp]-yph[ip],2));
                    frh=-mp[ip]/pow(rph[jp][ip],2);
                    fxh[jp][ip]=frh*(xph[jp]-xph[ip])/rph[jp][ip];
                    fyh[jp][ip]=frh*(yph[jp]-yph[ip])/rph[jp][ip];

                }else{
                    rph[jp][ip]=0;
                    fxh[jp][ip]=0;
                    fyh[jp][ip]=0;
                }
            }
        }

        for(int ip=0; ip<3; ip++){
            double fxtotal, fytotal;
            fxtotal=fytotal=0;

            for(int jp=0; jp<3; jp++){
                fxtotal+=fxh[ip][jp];
             
                fytotal+=fyh[ip][jp];
            }

            xp[ip]=xp[ip] + vxh[ip]*dt;
            yp[ip]=yp[ip] + vyh[ip]*dt;
            vx[ip]=vx[ip]+fxtotal*dt;
            vy[ip]=vy[ip]+fytotal*dt;            

        }

        time+=dt;

        if(step%10000==1){
            //std::cout << step << ":" << xp[1] << "," << yp[1] << std::endl;
	      
            if(count < 10000){
		 for(int ip=0; ip<3; ip++){
                    data_list[count].xp[ip]=xp[ip];
                    data_list[count].yp[ip]=yp[ip];
		      data_list[count].ag[ip]= sqrt(xp[ip]*xp[ip] + yp[ip]*yp[ip]) * sqrt(vx[ip]*vx[ip] + vy[ip]*vy[ip]);              
		  }
             }
            count++;
        }


     }
	
     CalcStDeviation();
}

//////////////////////////////////////////////
//4次ルンゲクッタ
//
//////////////////////////////////////////////

CThreeBodyRK4::CThreeBodyRK4()
:CCalcEngine()
{
}

CThreeBodyRK4::~CThreeBodyRK4()
{
}

//ルンゲクッタ4次によるソルバ
void CThreeBodyRK4::Calculation()
{
    double fr, fx[3][3], fy[3][3];
    double rp[3][3];
    double dth, frh, xph[3], yph[3], rph[3][3], vxh[3], vyh[3], fxh[3][3], fyh[3][3];
    double k1x[3],k1y[3], k2x[3],k2y[3], k3x[3],k3y[3], k4x[3], k4y[3];
    dt=0;
    count=0;

    for(int step=1; step < nstep; step++){
        dtmin=1.0e11;

        for(int ip=0; ip<3; ip++){
            for(int jp=0; jp<3; jp++){
                if(jp != ip){
                    rp[jp][ip]=sqrt(pow(xp[jp]-xp[ip],2)+pow(yp[jp]-yp[ip],2));
                    fr=-mp[ip]/pow(rp[jp][ip],2);
                    fx[jp][ip]=fr*(xp[jp]-xp[ip])/rp[jp][ip];
                    fy[jp][ip]=fr*(yp[jp]-yp[ip])/rp[jp][ip];
                    vp = sqrt(pow(vx[jp]-vx[ip],2)+pow(vy[jp]-vy[ip],2));
                    dtmin=min(dtmin, rp[jp][ip]/vp);

                }else{
                    rp[jp][ip]=0;
                    fx[jp][ip]=0;
                    fy[jp][ip]=0;
                }
            }
        }


        dt=2*PI_*dtmin*1.001e-3;
        dth = dt*0.5e0;

        for(int ip=0; ip<3; ip++){
            double fxtotal, fytotal;
            fxtotal=fytotal=0;

            for(int jp=0; jp<3; jp++){
               fxtotal+=fx[ip][jp];
                fytotal+=fy[ip][jp]; 
            }

            k1x[ip]=fxtotal;
            k1y[ip]=fytotal;
	     
	     xph[ip]=xp[ip] + k1x[ip]*dt/2;
            yph[ip]=yp[ip] + k1y[ip]*dt/2;
		
	     
        }

	//k2
        for(int ip=0; ip<3; ip++){
            for(int jp=0; jp<3; jp++){
                if(jp != ip){
                    rph[jp][ip]=sqrt(pow(xph[jp]-xph[ip],2)+pow(yph[jp]-yph[ip],2));
                    frh=-mp[ip]/pow(rph[jp][ip],2);
                    fxh[jp][ip]=frh*(xph[jp]-xph[ip])/rph[jp][ip];
                    fyh[jp][ip]=frh*(yph[jp]-yph[ip])/rph[jp][ip];

                }else{
                    rph[jp][ip]=0;
                    fxh[jp][ip]=0;
                    fyh[jp][ip]=0;
                }
            }
        }

        for(int ip=0; ip<3; ip++){
            double fxtotal, fytotal;
            fxtotal=fytotal=0;

            for(int jp=0; jp<3; jp++){
                fxtotal+=fxh[ip][jp];
             
                fytotal+=fyh[ip][jp];
            }

            k2x[ip]=fxtotal;
            k2y[ip]=fytotal;
	     
	     xph[ip]=xp[ip] + k2x[ip]*dt/2;
            yph[ip]=yp[ip] + k2y[ip]*dt/2;
        }

	//k3
	for(int ip=0; ip<3; ip++){
            for(int jp=0; jp<3; jp++){
                if(jp != ip){
                    rph[jp][ip]=sqrt(pow(xph[jp]-xph[ip],2)+pow(yph[jp]-yph[ip],2));
                    frh=-mp[ip]/pow(rph[jp][ip],2);
                    fxh[jp][ip]=frh*(xph[jp]-xph[ip])/rph[jp][ip];
                    fyh[jp][ip]=frh*(yph[jp]-yph[ip])/rph[jp][ip];

                }else{
                    rph[jp][ip]=0;
                    fxh[jp][ip]=0;
                    fyh[jp][ip]=0;
                }
            }
        }

        for(int ip=0; ip<3; ip++){
            double fxtotal, fytotal;
            fxtotal=fytotal=0;

            for(int jp=0; jp<3; jp++){
                fxtotal+=fxh[ip][jp];
             
                fytotal+=fyh[ip][jp];
            }

            k3x[ip]=fxtotal;
            k3y[ip]=fytotal;
	     
	     xph[ip]=xp[ip] + k3x[ip]*dt;
            yph[ip]=yp[ip] + k3y[ip]*dt;
        }
	
	//k4
	for(int ip=0; ip<3; ip++){
            for(int jp=0; jp<3; jp++){
                if(jp != ip){
                    rph[jp][ip]=sqrt(pow(xph[jp]-xph[ip],2)+pow(yph[jp]-yph[ip],2));
                    frh=-mp[ip]/pow(rph[jp][ip],2);
                    fxh[jp][ip]=frh*(xph[jp]-xph[ip])/rph[jp][ip];
                    fyh[jp][ip]=frh*(yph[jp]-yph[ip])/rph[jp][ip];

                }else{
                    rph[jp][ip]=0;
                    fxh[jp][ip]=0;
                    fyh[jp][ip]=0;
                }
            }
        }

        for(int ip=0; ip<3; ip++){
            double fxtotal, fytotal;
            fxtotal=fytotal=0;

            for(int jp=0; jp<3; jp++){
                fxtotal+=fxh[ip][jp];
             
                fytotal+=fyh[ip][jp];
            }

            k4x[ip]=fxtotal;
            k4y[ip]=fytotal;
	     
	     	vx[ip] = vx[ip] + (k1x[ip]/6 + k2x[ip]/3 + k3x[ip]/3 + k4x[ip]/6)*dt;
		vy[ip] = vy[ip] + (k1y[ip]/6 + k2y[ip]/3 + k3y[ip]/3 + k4y[ip]/6)*dt;
		xp[ip] = xp[ip] + dt*vx[ip];
		yp[ip] = yp[ip] + dt*vy[ip];
        }

        time+=dt;

        if(step%10000==1){
            //std::cout << step << ":" << xp[1] << "," << yp[1] << std::endl;
	      
            if(count < 10000){
		 for(int ip=0; ip<3; ip++){
                    data_list[count].xp[ip]=xp[ip];
                    data_list[count].yp[ip]=yp[ip];
		      data_list[count].ag[ip]= sqrt(xp[ip]*xp[ip] + yp[ip]*yp[ip]) * sqrt(vx[ip]*vx[ip] + vy[ip]*vy[ip]);              
		  }
             }
            count++;
        }


     }
	
     CalcStDeviation();
}




